package aQute.lib.struct;

import java.lang.reflect.*;
import java.util.*;

import aQute.lib.converter.*;
import aQute.lib.converter.Converter.Hook;
import aQute.struct.*;

@SuppressWarnings({
		"rawtypes", "unchecked"
})
public class StructUtil {
	static Converter	converter	= new Converter();
	static {
		/**
		 * We have to hook into the converter to ensure that different type
		 * structs can be copied to each other. Basically, whenever we find a
		 * conversion to a struct a struct, we take over.
		 */
		converter.hook(null, new Hook() {

			@Override
			public Object convert(Type dest, Object from) throws Exception {
				if (dest instanceof Class && struct.class.isAssignableFrom((Class) dest) && from instanceof struct) {
					struct to = (struct) ((Class) dest).newInstance();
					copy((struct) from, to);
					return to;
				}
				return null;
			}
		});
	}

	static public <T extends struct> T copy(struct from, T to, String... excludes) throws Exception {
		Arrays.sort(excludes);
		for (Field f : from.fields()) {
			if (Arrays.binarySearch(excludes, f.getName()) >= 0)
				continue;

			Object o = f.get(from);
			if (o == null)
				continue;

			Field tof = to.getField(f.getName());
			if (tof != null)
				try {
					tof.set(to, Converter.cnv(tof.getGenericType(), o));
				}
				catch (Exception e) {
					System.out.println("Failed to convert " + f.getName() + " from " + from.getClass() + " to "
							+ to.getClass() + " value " + o + " exception " + e);
				}
		}

		return to;
	}

	static public <T extends struct> T merge(struct from, T to, String... excludes) throws Exception {
		Arrays.sort(excludes);
		for (Field fromField : from.fields()) {
			if (Arrays.binarySearch(excludes, fromField.getName()) >= 0)
				continue;

			Object fromMember = fromField.get(from);
			if (fromMember == null)
				continue;

			Field toField = to.getField(fromField.getName());
			if (toField != null) {
				Object toMember = toField.get(to);
				Object fromMemberConverted = converter.convert(toField.getGenericType(), fromMember);

				if (unset(toMember))
					toField.set(to, fromMemberConverted);
				else if (toMember instanceof Collection) {
					Collection c = (Collection) fromMemberConverted;
					c.addAll((Collection) toMember);
					toField.set(to, c);
				} else if (toMember instanceof Map) {
					Map m = (Map) fromMemberConverted;
					m.putAll((Map) toMember);
					toField.set(to, m);
				} else {
					// Skip the field
				}
			}
		}

		return to;
	}

	public static boolean unset(Object object) {
		return object == null || (object instanceof Number && ((Number) object).longValue() == 0);
	}
}
