package aQute.library.remote;

import java.io.*;
import java.lang.reflect.*;
import java.net.*;
import java.util.*;
import java.util.Map.Entry;

import aQute.lib.converter.*;
import aQute.lib.io.*;
import aQute.lib.json.*;

@SuppressWarnings("unchecked")
public class URLClient {
	static JSONCodec	codec	= new JSONCodec();
	String				url		= "http://repo.jpm4j.org/rest/";
	List<String>		cookies	= new ArrayList<String>();

	public URLClient(String baseurl) {
		this.url = baseurl;
		if (!url.endsWith("/"))
			this.url = this.url + "/";
	}

	<X> X send(String verb, String path, Object to, TypeReference<X> ref, Map<String,List<String>> arguments)
			throws Exception {
		return (X) send(verb, path, to, ref.getType(), arguments);
	}

	<X> X send(String verb, String path, Object to, Class<X> ref, Map<String,List<String>> arguments) throws Exception {
		return (X) send(verb, path, to, (Type) ref, arguments);
	}

	Object send(String verb, String path, Object to, Type ref, Map<String,List<String>> arguments) throws Exception {
		StringBuilder sb = new StringBuilder(url);
		sb.append(path);
		if (arguments != null) {
			String del = "?";
			for (Entry<String,List<String>> e : arguments.entrySet()) {
				for (Object o : e.getValue()) {
					sb.append(del);
					del = "&";
					sb.append(e.getKey());
					sb.append('=');
					sb.append(URLEncoder.encode(o.toString(), "UTF-8"));
				}
			}
		}
		URL url = new URL(sb.toString());
		HttpURLConnection con = (HttpURLConnection) url.openConnection();
		con.setRequestMethod(verb);
		if (to != null)
			con.setDoOutput(true);

		// if (cookies != null) {
		// for (String cookie : cookies) {
		// // TODO do we need to filter here?
		// // TODO expire?
		// con.addRequestProperty("Cookie", cookie);
		// }
		// }

		sign(con);

		if (to != null) {
			OutputStream out = con.getOutputStream();
			if (to instanceof InputStream)
				IO.copy((InputStream) to, out);
			else if (to instanceof byte[])
				IO.copy((byte[]) to, out);
			else if (to instanceof File)
				IO.copy((File) to, out);
			else
				codec.enc().to(out).put(to).flush();
			out.close();
		}

		int result = con.getResponseCode();
		if (result / 100 != 2) {
			String s = "";
			InputStream in = con.getErrorStream();
			if (in != null)
				s = IO.collect(in);
			if (result == 404)
				return null;

			throw new Exception("Failed request " + result + ":" + con.getResponseMessage() + " " + s);
		}

		InputStream in = con.getInputStream();
		try {
			Map<String,List<String>> headerFields = con.getHeaderFields();
			List<String> l = headerFields.get("Set-Cookie");
			if (l != null)
				for (String c : l) {
					cookies.add(c);
				}

			String s = IO.collect(in);
			return codec.dec().from(s).get(ref);
		}
		finally {
			in.close();
		}

	}

	protected void sign(HttpURLConnection con) throws Exception {}

	public void put(String path, Object to) throws Exception {
		send("PUT", path, to, (Type) null, null);
	}

	public <T> T get(String path, Type type, Map<String,List<String>> args) throws Exception {
		return (T) send("GET", path, null, type, args);
	}

}
