package aQute.library.util;

import java.io.*;
import java.net.*;

import javax.xml.parsers.*;
import javax.xml.xpath.*;

import org.w3c.dom.*;

import aQute.service.library.*;

public class OBRBatch extends StageBatch {

	public OBRBatch(Library lib) {
		super(lib);
	}

	static DocumentBuilderFactory	dbf	= DocumentBuilderFactory.newInstance();
	static XPathFactory				xpf	= XPathFactory.newInstance();
	{
		dbf.setNamespaceAware(false);
	}

	public void parse(URL url) throws Exception {
		DocumentBuilder db = dbf.newDocumentBuilder();
		XPath xp = xpf.newXPath();
		InputStream in = url.openStream();
		try {
			Document doc = db.parse(in);
			String xlmns = xp.evaluate("repository/@xmlns", doc);
			if (xlmns != null && xlmns.length() > 0) {
				parseRepository1_0_0(url, xp, doc);
			} else {
				parseRfc112(url, xp, doc);
			}
		}
		finally {
			in.close();
		}
	}

	private void parseRfc112(URL url, XPath xp, Document doc) throws XPathExpressionException {
		NodeList nodes = (NodeList) xp.evaluate("repository/resource", doc, XPathConstants.NODESET);
		for (int i = 0; i < nodes.getLength(); i++) {
			try {
				Node a = nodes.item(i);
				String value = xp.evaluate("./@uri", a);
				if (value == null) {
					error("Expected url, but got null %s", a);
					continue;
				}

				URI r = new URL(url, value).toURI();
				stage(r);
			}
			catch (Exception e) {
				exception(e, "Failed");
			}
		}
	}

	// TODO change to use the metadata and not collect the thingy remotely
	private void parseRepository1_0_0(URL url, XPath xp, Document doc) throws XPathExpressionException {
		Node root = doc.getDocumentElement();
		String name = xp.evaluate("./@name", root);
		String increment = xp.evaluate("./@increment", root);
		if (name != null) {
			message += " " + name;
		}

		if (increment != null) {
			message += "-" + increment;
		}
		super.message = message;

		NodeList nodes = (NodeList) xp.evaluate("resource/capability[@namespace='osgi.content']", root,
				XPathConstants.NODESET);
		for (int i = 0; i < nodes.getLength(); i++) {
			try {

				Node a = nodes.item(i);
				String sha = xp.evaluate("attribute[@name='osgi.content']/@value", a);
				String path = xp.evaluate("attribute[@name='url']/@value", a);

				if (sha == null) {
					error("Expected sha, but got null %s", a);
					continue;
				}

				if (path == null) {
					error("Expected path, but got null %s", a);
					continue;
				}

				URI r = new URL(url, path).toURI();
				stage(r);
			}
			catch (Exception e) {
				exception(e, "Failed");
			}
		}
	}
}
