package aQute.library.util;

import java.io.*;
import java.net.*;
import java.util.*;

import aQute.libg.reporter.*;
import aQute.maven.*;
import aQute.service.library.*;

public class StageBatch extends ReporterAdapter implements Closeable {
	final Library					lib;
	final List<Library.Revision>	responses	= new ArrayList<Library.Revision>();
	String							message;
	String							owner;
	Throwable						abort;
	List<String>					tos			= new ArrayList<String>();
	boolean							mail		= true;
	private String					request;

	public StageBatch(Library lib) {
		this.lib = lib;
	}

	public StageBatch message(String message) {
		return this;
	}

	public StageBatch owner(String owner) {
		this.owner = owner;
		return this;
	}

	public StageBatch stage(URI uri) {
		try {
			trace("scanning %s", uri);
			Library.Revision scan = lib.scan(uri);
			Pom pom = MavenRepository.getPomFromArtifactUrl(uri);
			if (pom != null)
				PomUtil.merge(scan, pom);

			addErrors(uri.toString(), scan.errors);
			addWarnings(uri.toString(), scan.warnings);

			String id = scan.bsn + "-" + scan.version;
			trace("staging %s", id);
			Library.Revision response = lib.stage(scan);
			addErrors(id, response.errors);
			addWarnings(id, response.warnings);
			responses.add(response);
		}
		catch (Exception e) {
			exception(e, "staging %s %s", uri);
		}
		return this;
	}

	public StageBatch trace() {
		setTrace(true);
		return this;
	}

	public String toHtml() {
		// TODO Auto-generated method stub
		return null;
	}

	public String toText() {
		StringBuilder sb = new StringBuilder();
		Formatter format = new Formatter(sb);
		format.format("Imported at %s %s %s\n\n", new Date(), owner, message);
		format.format("%-40s %1s %-15s %-20s %6s %s\n", "Symbolic Name", "M", "Version", "Qualifier", "Errors",
				"Summary");

		for (Library.Revision r : responses) {
			if (r != null) {
				format.format("%-40s %1s %-15s %-20s %6d %s\n", //
						r.bsn, //
						r.phase, //
						r.baseline, //
						r.qualifier == null ? "" : r.qualifier, //
						r.errors.size(), //
						r.releaseSummary == null ? "" : r.releaseSummary);
			}
		}
		report(sb);
		if (request != null) {
			sb.append("---------------------\n");
			sb.append(request);
		}

		return sb.toString();
	}

	public void abort(Throwable e) {
		exception(e, "Aborted");
	}

	public void close() throws IOException {}

	public StageBatch request(String s) {
		this.request = s;
		return this;
	}

}
