package aQute.maven;

import java.io.*;
import java.net.*;
import java.util.*;
import java.util.regex.*;

import aQute.lib.io.*;

class HtmlPage implements Iterable<HtmlPage> {
	final static Pattern	LINK	= Pattern.compile("<a[^>]href\\s*=\\s*(:?'|\")?([^\"'>\\s]+)(:?'|\")?(:?\\s|>)",
											Pattern.CASE_INSENSITIVE + Pattern.MULTILINE);
	final String			base;
	final List<String>		links	= new ArrayList<String>();
	final String			pom;

	public HtmlPage(String base) throws IOException {
		if (!base.endsWith("/"))
			base += "/";
		this.base = base;

		URL url = new URL(base);
		String page = IO.collect(url.openStream());
		Matcher m = LINK.matcher(page);
		String pom = null;

		while (m.find()) {
			String link = m.group(2);
			if (link.startsWith("."))
				continue;

			if (link.startsWith("maven-metadata"))
				continue;

			if (link.endsWith(".sha1") || link.endsWith(".md5"))
				continue;

			if (link.endsWith(".pom"))
				pom = link;
			else
				links.add(link);
		}
		this.pom = pom;
		Collections.sort(links);
	}

	public List<String> getLinks() {
		return links;
	}

	public boolean isPom() {
		return pom != null;
	}

	public String getPom() {
		if (isPom())
			return base + pom;
		else
			return null;
	}

	public Iterator<HtmlPage> iterator() {
		return new HtmlPageIterator(this, 0);
	}

	public HtmlPage getPage(String link) throws IOException {
		return new HtmlPage(base + link);
	}

}
