package aQute.struct;

import java.lang.annotation.*;

/**
 * Define the semantics of this field. Can be used by the program to provide
 * information about a field
 */
@Retention(RetentionPolicy.RUNTIME)
@Target({
	ElementType.FIELD
})
public @interface Define {
	/**
	 * Human readable description of this field in English. Localization
	 * techniques should use the class.field ide to provide localized
	 * descriptions if needed.
	 * 
	 * @return the description, can be the empty string
	 */
	String description() default "";

	/**
	 * If true, this field is required to be not null (default).
	 * 
	 * @return if this field is required
	 */
	boolean optional() default false;

	/**
	 * If set, checks if this field should be regarded as a field to be
	 * considered in equals.
	 * 
	 * @return
	 */
	boolean id() default false;

	/**
	 * Regular expression patterns that must all match.
	 * 
	 * @return
	 */
	String pattern() default "";

	/**
	 * For numeric fields, check if they are more or equals than min
	 */
	long min() default 0;

	/**
	 * For numeric fields, check if they are less or equals than max
	 */
	long max() default Integer.MAX_VALUE;

}
